@@ -5,6 +5,8 @@ Web application
 import webapi as web
 import webapi, wsgi, utils
 import debugerror
+import httpserver
+
 from utils import lstrips, safeunicode
 import sys
 
@@ -201,7 +203,7 @@ class application:
         if 'HTTP_CONTENT_TYPE' in env:
             env['CONTENT_TYPE'] = env.pop('HTTP_CONTENT_TYPE')
 
-        if method in ["POST", "PUT"]:
+        if method not in ["HEAD", "GET"]:
             data = data or ''
             import StringIO
             if isinstance(data, dict):
@@ -309,6 +311,13 @@ class application:
         function.
         """
         return wsgi.runwsgi(self.wsgifunc(*middleware))
+
+    def stop(self):
+        """Stops the http server started by run.
+        """
+        if httpserver.server:
+            httpserver.server.stop()
+            httpserver.server = None
     
     def cgirun(self, *middleware):
         """
@@ -325,7 +334,49 @@ class application:
         except ImportError:
             # we're not running from within Google App Engine
             return wsgiref.handlers.CGIHandler().run(wsgiapp)
-    
+
+    def gaerun(self, *middleware):
+        """
+        Starts the program in a way that will work with Google app engine,
+        no matter which version you are using (2.5 / 2.7)
+
+        If it is 2.5, just normally start it with app.gaerun()
+
+        If it is 2.7, make sure to change the app.yaml handler to point to the
+        global variable that contains the result of app.gaerun()
+
+        For example:
+
+        in app.yaml (where code.py is where the main code is located)
+
+            handlers:
+            - url: /.*
+              script: code.app
+
+        Make sure that the app variable is globally accessible
+        """
+        wsgiapp = self.wsgifunc(*middleware)
+        try:
+            # check what version of python is running
+            version = sys.version_info[:2]
+            major   = version[0]
+            minor   = version[1]
+
+            if major != 2:
+                raise EnvironmentError("Google App Engine only supports python 2.5 and 2.7")
+
+            # if 2.7, return a function that can be run by gae
+            if minor == 7:
+                return wsgiapp
+            # if 2.5, use run_wsgi_app
+            elif minor == 5:
+                from google.appengine.ext.webapp.util import run_wsgi_app
+                return run_wsgi_app(wsgiapp)
+            else:
+                raise EnvironmentError("Not a supported platform, use python 2.5 or 2.7")
+        except ImportError:
+            return wsgiref.handlers.CGIHandler().run(wsgiapp)
+     
     def load(self, env):
         """Initializes ctx using env."""
         ctx = web.ctx
@@ -366,8 +417,10 @@ class application:
         ctx.fullpath = ctx.path + ctx.query
         
         for k, v in ctx.iteritems():
+            # convert all string values to unicode values and replace 
+            # malformed data with a suitable replacement marker.
             if isinstance(v, str):
-                ctx[k] = safeunicode(v)
+                ctx[k] = v.decode('utf-8', 'replace') 
 
         # status must always be str
         ctx.status = '200 OK'
@@ -636,9 +689,12 @@ class Reloader:
     if so, reloads them.
     """
 
-    SUFFIX = '$py.class' if sys.platform.startswith('java') else '.pyc'
     """File suffix of compiled modules."""
-
+    if sys.platform.startswith('java'):
+        SUFFIX = '$py.class'
+    else:
+        SUFFIX = '.pyc'
+    
     def __init__(self):
         self.mtimes = {}
 
