@@ -131,8 +131,14 @@ def storify(mapping, *requireds, **defaults):
         <Storage {'x': u'a'}>
     """
     _unicode = defaults.pop('_unicode', False)
+
+    # if _unicode is callable object, use it convert a string to unicode.
+    to_unicode = safeunicode
+    if _unicode is not False and hasattr(_unicode, "__call__"):
+        to_unicode = _unicode
+    
     def unicodify(s):
-        if _unicode and isinstance(s, str): return safeunicode(s)
+        if _unicode and isinstance(s, str): return to_unicode(s)
         else: return s
         
     def getvalue(x):
@@ -639,6 +645,15 @@ class IterBetter:
             ...
         IndexError: already passed 3
 
+    It is also possible to get the first value of the iterator or None.
+
+        >>> c = iterbetter(iter([3, 4, 5]))
+        >>> print c.first()
+        3
+        >>> c = iterbetter(iter([]))
+        >>> print c.first()
+        None
+
     For boolean test, IterBetter peeps at first value in the itertor without effecting the iteration.
 
         >>> c = iterbetter(iter(range(5)))
@@ -655,6 +670,18 @@ class IterBetter:
     def __init__(self, iterator): 
         self.i, self.c = iterator, 0
 
+    def first(self, default=None):
+        """Returns the first element of the iterator or None when there are no
+        elements.
+
+        If the optional argument default is specified, that is returned instead
+        of None when there are no elements.
+        """
+        try:
+            return iter(self).next()
+        except StopIteration:
+            return default
+
     def __iter__(self): 
         if hasattr(self, "_head"):
             yield self._head
@@ -715,7 +742,7 @@ def safewrite(filename, content):
     f = file(filename + '.tmp', 'w')
     f.write(content)
     f.close()
-    os.rename(f.name, path)
+    os.rename(f.name, filename)
 
 def dictreverse(mapping):
     """
@@ -905,7 +932,12 @@ def datestr(then, now=None):
         if abs(deltadays) < 4:
             return agohence(deltadays, 'day')
 
-        out = then.strftime('%B %e') # e.g. 'June 13'
+        try:
+            out = then.strftime('%B %e') # e.g. 'June  3'
+        except ValueError:
+            # %e doesn't work on Windows.
+            out = then.strftime('%B %d') # e.g. 'June 03'
+
         if then.year != now.year or deltadays < 0:
             out += ', %s' % then.year
         return out
@@ -1199,7 +1231,7 @@ class ThreadedDict(threadlocal):
     def clear_all():
         """Clears all ThreadedDict instances.
         """
-        for t in ThreadedDict._instances:
+        for t in list(ThreadedDict._instances):
             t.clear()
     clear_all = staticmethod(clear_all)
     
@@ -1482,7 +1514,7 @@ class _EmailMessage:
             c = boto.ses.SESConnection(
               aws_access_key_id=webapi.config.get('aws_access_key_id'),
               aws_secret_access_key=web.api.config.get('aws_secret_access_key'))
-            c.send_raw_email(self.from_address, message_text, self.from_recipients)
+            c.send_raw_email(self.from_address, message_text, self.recipients)
         else:
             sendmail = webapi.config.get('sendmail_path', '/usr/sbin/sendmail')
         
